//device.c: usb functions using libusb
//Copyright 2006 Jeroen Domburg <s1fwx@jeroen.ietsmet.nl>
//This software is licensed under the GNU GPL.

#include "device.h"
USB_HANDLE device_open(char drive) {
    struct usb_bus *busses;
    struct usb_bus *bus;
    USB_HANDLE device_handle;
    int r=0;
    device_handle = NULL;
    
    usb_init();
    usb_find_busses();
    usb_find_devices();
    

    busses = usb_get_busses();
    for (bus = busses; bus; bus = bus->next) {
    	struct usb_device *dev;
    	for (dev = bus->devices; dev; dev = dev->next) {
	    if ((dev->descriptor.idVendor==4310)&&(dev->descriptor.idProduct==65361) && (!device_handle)) {
	        printf("device_lin: device found\n");
	        device_handle=usb_open(dev);
//ricardol:The error control logic is not tested
//ricardol:The only two cases tested are when there is not libusb devices present
//ricardol:and when there is one. 
                if (device_handle==NULL) {
        	    printf("Could not get device handler\n");
		    return(0);
        	} 
//ricardol:i put this just in case, but it returns an error, 
//                     r=usb_set_altinterface(device_handle, 0);
//			         if (r<0) perror("Set_altinterface if\n");
//JeroenD: Linux works OK without this, but fscks up the write with this. Weird.
#ifndef LINUX
//ricardol: Looks like this has to be done under windows 
//ricardol: if interface claiming is needed
//                r=usb_set_configuration(device_handle, 1);
			         if (r<0) perror("Set_configuration if\n");
#endif
                r=r+usb_claim_interface(device_handle, 0);
		if (r!=0) {
                    perror("Claim or configuration if\n");
                    usb_close(device_handle);
                    device_handle=NULL;
                }            
            }	 			         
	}
    }
    return device_handle;
}


// -----------------------------------------------------------------------------------------------
bool device_close(USB_HANDLE hd)
{
    return usb_close(hd);
}


// -----------------------------------------------------------------------------------------------
long device_send(USB_HANDLE hd, void *data, uint32 max_data)
{
    return usb_bulk_write(hd, 1, (char *)data, max_data, 1024);
}


static void printbuff(char* buff, int len) {
    int x;
    for (x=0; x<len; x++) {
	printf("%02hhx ",buff[x]);
	if ((x&15)==15) printf("\n");
    }
    printf("\n");
}

// -----------------------------------------------------------------------------------------------
long device_send_cmd(USB_HANDLE device_handle, LP_CMD cmd, void *data, uint32 max_data)
{
    uint8 tx_buf[CMD_BUFSIZE];
    uint32 size=0;
    int r;
  
    fprintf(stderr,"Send cmd 0x%x\n", cmd->cmd);
    //refill cmd
    cmd->size=max_data;
    cmd->undef0x04=0xEFBEADDE; //0xDEADBEEF as tag, gets noticed in USB dumps :)
    //Standard CBW header
    cmd->id[0]='U';cmd->id[1]='S';
    cmd->id[2]='B';cmd->id[3]='C';
  
    //Write the data
    fprintf(stderr,"Write cmd, 0x1f bytes.\n");
    printbuff((char*)cmd,0x1f);
    r=usb_bulk_write(device_handle, 1, (char*)cmd, 0x1f, 100);
    if (r<0) {
	perror("Error @ write cbw");
	return(-1);
    }
    
    if (max_data>0) {
	if(cmd->mode == 0x80) {   // read data...
    	    fprintf(stderr,"Read data, 0x%x bytes\n",max_data);
    	    r=usb_bulk_read(device_handle, 0x82, (char*)data, max_data, 1000);
    	    if (r<0) {
    		perror("Read data"); 
		return(-1);
    	    }
    	    size=r;
	} else {                  // send data...
    	    fprintf(stderr,"Write data, 0x%x bytes\n",max_data);
    	    r=usb_bulk_write(device_handle, 1, (char*)data,  max_data, 1000);
    	    if (r<0) {
    		perror("Write data");
		return(-1);
    	    }
	}
	fprintf(stderr,"Done -- 0x%x bytes.\n",r);
    }
    //Read csw
    fprintf(stderr,"Read csw\n");
    r=usb_bulk_read(device_handle, 0x82, (char*)tx_buf, 0x0d, 100);
    printf("Got 0x%x bytes.\n",r);
    fprintf(stderr,"csw=%s\n",tx_buf);
    if (r<0) {
	perror("Error at read csw");
	return(-1);
    }
    return (long)size;
}

